////////////////////////////////////////////////////////////////////////////////////////////////
//                                     GAME EXTRACTOR                                         //
//                            Multi-Platform Game Archive Viewer                              //
//                               http://www.watto.org/extract                                 //
//                                                                                            //
//                   Game Extractor is Copyright (C) 2002-2006  WATTO Studios                 //
////////////////////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////////////////////
// This program is free software; you can redistribute it and/or modify it under the terms of //
// the GNU General Public License published by the Free Software Foundation; either version 2 //
// of the License, or (at your option) any later versions. This program is distributed in the //
// hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranties //
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License //
// at http://www.gnu.org for more details. For further information relating to Game Extractor //
// including program updates and plugins, visit http://www.watto.org/extract . Thank you!! :) //
////////////////////////////////////////////////////////////////////////////////////////////////

import java.io.File;

/**
**********************************************************************************************

**********************************************************************************************
**/
public class Plugin_RCF_ATGCORE_BE extends ArchivePlugin {


/**
**********************************************************************************************

**********************************************************************************************
**/
  public Plugin_RCF_ATGCORE_BE() {

    name = "RCF_ATGCORE_BE";
    canRead = true;
    canWrite = false;
    canReplace = false;
    canRename = false;

    games = new String[]{"The Incredible Hulk - Ultimate Destruction (GC)"};
    extensions = new String[]{"rcf"};

    columns = Archive.getColumns();

    //fileTypes = new String[][]{{"",""},
    //                           {"",""},
    //                           {"",""}};

    }


/**
**********************************************************************************************

**********************************************************************************************
**/
  public int getMatchRating(FileManipulator fm) {
    try {

      int rating = 0;

      if (fm.getExtension().equals(extensions[0])){
        rating += 25;
        }

      // Header
      if (fm.readString(23).equals("ATG CORE CEMENT LIBRARY")){
        rating += 50;
        }

      fm.skip(13);

      // Directory Offset
      if (fm.readIntB() == 60){
        rating += 5;
        }

      int arcSize = fm.getLength();

      // Directory Length
      if (check.length(fm.readIntB(),arcSize)){
        rating += 5;
        }

      fm.skip(12);

      // Number Of Files
      if (check.numFiles(fm.readIntB())){
        rating += 5;
        }

      return rating;

      }
    catch (Throwable t){
      return 0;
      }
    }


/**
**********************************************************************************************
Reads an [archive] File into the Resources
**********************************************************************************************
**/
  public Resource[] read(File path) {
    try {

      // NOTE - Compressed files MUST know their DECOMPRESSED LENGTH
      //      - Uncompressed files MUST know their LENGTH

      addFileTypes();
      //ExporterPlugin exporter = Exporter_ZLib.getInstance();

      // RESETTING THE GLOBAL VARIABLES


      FileManipulator fm = new FileManipulator(path,"r");

      int arcSize = (int)fm.getLength();

      // 32 - Header ("ATG CORE CEMENT LIBRARY" + nulls to fill)
      // 4 - Unknown
      fm.skip(36);

      // 4 - Directory Offset (60)
      int dirOffset = fm.readIntB();
      check.offset(dirOffset,arcSize);

      // 4 - Directory Length
      fm.skip(4);

      // 4 - Offset To Filename Directory
      int filenameDirOffset = fm.readIntB();
      check.offset(filenameDirOffset,arcSize);

      // 4 - Filename Directory Length
      // 4 - null
      fm.skip(8);

      // 4 - Number Of Files
      int numFiles = fm.readIntB();
      check.numFiles(numFiles);

      fm.seek(filenameDirOffset + 8);

      // Loop through directory
      String[] names = new String[numFiles];
      for(int i=0;i<numFiles;i++){
        // 4 - Unknown
        // 4 - File Data Padding Multiple (2048)
        // 4 - null
        fm.skip(12);

        // 4 - Filename Length (including null terminator)
        int filenameLength = fm.readIntL() - 1;
        check.filenameLength(filenameLength);

        // X - Filename
        names[i] = fm.readString(filenameLength);

        // 4 nulls
	fm.skip(4);

        }


      Resource[] resources = new Resource[numFiles];

      WSProgressDialog.loadProgress(numFiles);



      fm.seek(dirOffset);

      // Loop through directory
      for(int i=0;i<numFiles;i++){
        // 4 - Hash?
        fm.skip(4);

        // 4 - File Offset
        int offset = fm.readIntB();
        check.offset(offset,arcSize);

        // 4 - File Length
        int length = fm.readIntB();
        check.length(length,arcSize);

        String filename = names[i];

        //path,id,name,offset,length,decompLength,exporter
        resources[i] = new Resource(path,filename,offset,length);

        WSProgressDialog.setProgress(i);
        }



      fm.close();

      return resources;

      }
    catch (Throwable t){
      logError(t);
      return null;
      }
    }


  }

